{
   frmuMain, Main form
   Copyright (c) 2011. Dejan Boras

   This file is part of OpenArena Name Changer (oanc).

   domTrack is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   domTrack is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with domTrack. If not, see <http://www.gnu.org/licenses/>.      
   
   Started On:    01.03.2011.
   Last Update:   09.03.2011.
}

{$MODE OBJFPC}{$H+}
UNIT frmuMain;

INTERFACE

USES
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Buttons, ExtCtrls, BGRABitmap, BGRABitmapTypes,
  {oanc}
  frmuAbout, frmuFav, frmuCfg, frmuModSelect, uoanc;

TYPE

  { TfrmMain }

  TfrmMain = class(TForm)
     btnFav: TBitBtn;
     btnAddToFav: TBitBtn;
     btnName: TBitBtn;
     btnClear: TBitBtn;
     btnQuit: TBitBtn;
     btnAbout: TBitBtn;
     btnSaveSel: TBitBtn;
     btnSaveCfg: TBitBtn;
     clr1: TShape;
     clr2: TShape;
     clr3: TShape;
     clr4: TShape;
     clr5: TShape;
     clr6: TShape;
     clr7: TShape;
     clr8: TShape;
     edName: TEdit;
     lblEnterName: TLabel;
     clr0: TShape;
     shpChars: TShape;
     shpPrv: TShape;
     procedure btnClearClick(Sender: TObject);
     procedure btnAddToFavClick(Sender: TObject);
     procedure btnFavClick(Sender: TObject);
     procedure btnNameClick(Sender: TObject);
     procedure btnQuitClick(Sender: TObject);
     procedure btnAboutClick(Sender: TObject);
     procedure btnSaveCfgClick(Sender: TObject);
     procedure btnSaveSelClick(Sender: TObject);
     procedure clr0MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr1MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr2MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr3MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr4MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr5MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr6MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr7MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure clr8MouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure edNameChange(Sender: TObject);
     procedure shpCharsMouseUp(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Integer);
     procedure shpCharsPaint(Sender: TObject);
     procedure shpPrvPaint(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

VAR
  frmMain: TfrmMain;

IMPLEMENTATION

{$R *.lfm}

procedure Preview();
begin
   PreviewName(playername, frmMain.shpPrv);
end;

{set the entry}
procedure SetEntry(const s: string);
begin
   frmMain.edName.Text := '';
end;

procedure InsertText(const s: string);
var
   t: string;
   sel: longint;

begin
   t := frmMain.edName.Text;

   sel := frmMain.edName.SelStart;

   {insert into text}
   insert(s, t, sel+1);
   frmMain.edName.Text := t;

   {restore selection}
   frmMain.edName.SelStart := sel+length(s);
end;

procedure ClearEntry();
begin
   SetEntry('');
end;

{ TfrmMain }

procedure TfrmMain.btnQuitClick(Sender: TObject);
begin
   frmMain.Close();
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
   frmAbout.ShowModal();
end;

procedure TfrmMain.btnSaveCfgClick(Sender: TObject);
begin
   if(OAConfigSave(oacfgfile)) then
      MessageDlg('Success', 'OpenArena configuration file saved successfully.', mtInformation, [mbOK], 0);
end;

procedure TfrmMain.btnSaveSelClick(Sender: TObject);
begin
   if(playername <> '') then
      frmModSelect.ShowModal()
   else
      MessageDlg('Player name error', 'Cannot save an empty player name.',
         mtWarning, [mbOK], 0);
end;

procedure TfrmMain.edNameChange(Sender: TObject);
begin
   playername := edName.Text;
   frmMain.shpPrv.Invalidate;
   Preview();
end;

procedure TfrmMain.shpCharsMouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
var
   c: char;
   cx, cy: longint;

begin
   if(Button = mbLeft) then begin
      cx := x div CHAR_WIDTH;
      cy := y div CHAR_HEIGHT;

      c := char((cy * HCHARS) + cx);
      if(byte(c) <= MAX_CHAR) then InsertText(c);
   end;
end;

procedure TfrmMain.shpPrvPaint(Sender: TObject);
begin
   Preview();
end;

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
   ClearEntry();
end;

procedure TfrmMain.btnAddToFavClick(Sender: TObject);
begin
   if(playername <> '') then begin
      frmuFav.AddToFav(playername);
      MessageDlg('Success', 'Name added to favorites.', mtInformation, [mbOK], 0);
   end else
      MessageDlg('Error', 'Cannot add a empty name to favorites.', mtWarning, [mbOK], 0);
end;

procedure TfrmMain.btnFavClick(Sender: TObject);
begin
   frmFav.ShowModal();
   frmMain.edName.Text := playername;
end;

procedure TfrmMain.btnNameClick(Sender: TObject);
begin
   frmCfg.ShowModal();
end;

{ COLORS }
procedure InsertColor(clr: longint);
var
   clrs: string;

begin
   str(clr, clrs);
   InsertText('^'+clrs);
end;


procedure TfrmMain.clr0MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(0);
end;

procedure TfrmMain.clr1MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(1);
end;

procedure TfrmMain.clr2MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(2);
end;

procedure TfrmMain.clr3MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(3);
end;

procedure TfrmMain.clr4MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(4);
end;

procedure TfrmMain.clr5MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(5);
end;

procedure TfrmMain.clr6MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(6);
end;

procedure TfrmMain.clr7MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(7);
end;

procedure TfrmMain.clr8MouseUp(Sender: TObject; Button: TMouseButton;
   Shift: TShiftState; X, Y: Integer);
begin
   InsertColor(8);
end;

{ CHARACTER SELECTION }

procedure TfrmMain.shpCharsPaint(Sender: TObject);
var
   c, x, y: longint;

begin
   {$IFDEF WINDOWS}
   charselimg.Draw(frmMain.shpChars.Canvas, 0, 0, false);
   {$ELSE}
   {for some reason, under GTK we cannot directly draw the created charsel image}
   for x := 0 to (HCHARS-1) do begin
      for y := 0 to (VCHARS-1) do begin
         c := y*HCHARS+x;
         if(c <= MAX_CHAR) then
            imgs[c].Draw(shpChars.Canvas, x*CHAR_WIDTH, y*CHAR_HEIGHT, false);
      end;
   end;
   {$ENDIF}
end;

END.

