{
   uoanc, base unit for OA Name Changer
   Copyright (c) 2011. Dejan Boras

   This file is part of OpenArena Name Changer (oanc).

   domTrack is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 2 of the License, or
   (at your option) any later version.

   domTrack is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with domTrack. If not, see <http://www.gnu.org/licenses/>.   
   
   Started On:    01.03.2011.
   Last Update:   11.03.2011.
}

{$MODE OBJFPC}{$H+}{$I-}
UNIT uoanc;

INTERFACE

   USES classes, sysutils, Graphics, ExtCtrls, BGRABitmap, BGRABitmapTypes,
   IniFiles, Dialogs;

CONST
   { characters }
   MAX_COLOR = 8;
   MAX_CHAR = 127;
   CHAR_WIDTH = 16;
   CHAR_HEIGHT = 16;

   HCHARS   = 16;
   VCHARS   = 8;

{ colors }

TYPE
   TClr = array[0..2] of byte;

CONST
   CLR_BLACK         = 0000;
   CLR_RED           = 0001;
   CLR_GREEN         = 0002;
   CLR_YELLOW        = 0003;
   CLR_BLUE          = 0004;
   CLR_AQUA          = 0005;
   CLR_PURPLE        = 0006;
   CLR_WHITE         = 0007;
   CLR_ORANGE        = 0008;

   colors: array[0..MAX_COLOR] of TClr =
    ((000, 000, 000), {black}
     (255, 000, 000), {red}
     (000, 255, 000), {green}
     (255, 255, 000), {yellow}
     (000, 000, 255), {blue}
     (000, 255, 255), {aqua}
     (255, 000, 255), {purple}
     (255, 255, 255), {the racist color}
     (255, 160, 000));{orange}

   { config }

   {configuration defaults}
   cfgFileName  = 'oanc.ini';
   cfgDirectory = '.oanc'+DirectorySeparator;
   lcfgFileName = 'writeconfighere';
   oaCfgFileName = 'q3config.cfg';

   {save config locally?}
   localconfig: boolean = false;

VAR
   {images}
   imgs: array[0..127] of TBGRABitmap;
   charselimg: TBGRABitmap;

   loadedname, playername: string; {player name}
   oacfgfile: string; {openarean config file}

   {configuration}
   localcfgpath: string;
   cfgpath: string;
   cfgfile: string;
   iniCfg: TIniFile;

   modlist: TStringList; {list of mods}
   modlistsel: TStringList; {list of selected mods}

   {callbacks for favorites configuration}
   LoadFavorites, SaveFavorites: TProcedure;

{ IMAGES }

procedure LoadImages();
procedure CreateCharSelectImage();


{ CONFIGURATION }
procedure InitializeConfig();
procedure LoadConfig();
procedure SaveConfig();

{returns the file name and path to the OA configuration file}
function oaGetCfgFile(): string;
{checks whether the oa config file exist at the specified location}
function oaCfgFileExists(const fn: string): boolean;
{check the set oa config}
procedure CheckOAConfig();
{load the OA config}
procedure OAConfigLoad();
{save playername to OA config}
function OAConfigSave(const fn: string): boolean;

{MODS}
procedure oaSaveToMods();
procedure oaScanMods();
function oaHasMods(): boolean;
function oaBasePath(): string;

{ RENDERING }
procedure PreviewName(const thename: string; const shp: TShape);

IMPLEMENTATION

{ IMAGES }

procedure LoadImages();
var
   i: longint;
   ns: string;

begin
   for i := 0 to 127 do begin
      str(i, ns);
      imgs[i] := TBGRABitmap.Create('chars'+DirectorySeparator+ns+'.png');
   end;
end;

procedure CreateCharSelectImage();
var
   c, x, y: longint;

begin
   charselimg := TBGRABitmap.Create(HCHARS*CHAR_WIDTH, VCHARS*CHAR_WIDTH);

   for x := 0 to (HCHARS-1) do begin
      for y := 0 to (VCHARS-1) do begin
         c := y*HCHARS+x;
         if(c <= MAX_CHAR) then
            imgs[c].Draw(charselimg.Canvas, x*CHAR_WIDTH, y*CHAR_HEIGHT, false);
      end;
   end;
end;

{ COFNIGURATION }

procedure InitializeConfig();
var
   i: longint;

begin
   if(FileExists(localcfgpath+lcfgFileName)) then begin
      localconfig := true;
      cfgFile := localcfgpath + cfgFileName;

      exit;
   end;

   {$IFDEF WINDOWS}
      cfgpath := ExcludeTrailingPathDelimiter(GetEnvironmentVariable('APPDATA'));
   {$ELSE}
      {$IFDEF UNIX}
         cfgpath := ExcludeTrailingPathDelimiter(GetEnvironmentVariable('HOME'));
      {$ELSE}
         {$IFDEF DARWIN}
            {TODO: Need to check if this is valid for darwin(mac os x)}
            cfgpath := ExcludeTrailingPathDelimiter(GetEnvironmentVariable('HOME'));
         {$ELSE}
            cfgpath := ExcludeTrailingPathDelimiter(GetAppConfigDir(true));

            {remove the last path name so we can decide what config directory to use}
            if(length(cfgpath) > 1) then
            for i := length(cfgpath) downto 1 do begin
               if(cfgpath[i] = DirectorySeparator) then begin
                  delete(cfgpath, i, 8192); break;
               end;
            end;
         {$ENDIF}
      {$ENDIF}
   {$ENDIF}

   {determine config path and file}
   cfgpath := cfgpath+DirectorySeparator+cfgDirectory;
   cfgfile := cfgpath+cfgFileName;

   {create directories}
   mkdir(ExcludeTrailingPathDelimiter(cfgpath));
   IOResult();
end;

procedure LoadConfig();
begin
   iniCfg := TIniFile.Create(cfgfile, true);

   loadedname := iniCfg.ReadString('General', 'Name', 'player');
   oacfgfile := iniCfg.ReadString('General', 'OAConfigFile', '');

   if(LoadFavorites <> nil) then
      LoadFavorites();

   iniCfg.Free();
end;

procedure SaveConfig();
begin
   iniCfg := TIniFile.Create(cfgfile, true);

   iniCfg.WriteString('General', 'Name', playername);
   iniCfg.WriteString('General', 'OAConfigFile', oacfgfile);

   if(SaveFavorites <> nil) then
      SaveFavorites();

   iniCfg.Free();
end;

function oaGetCfgFile(): string;
var
   path: string;

begin
   path := '';
   {$IFDEF WINDOWS} {MS-WINDOWS}
   path := IncludeTrailingPathDelimiter(GetEnvironmentVariable('APPDATA')) +
      'OpenArena' + DirectorySeparator +
      'baseoa' + DirectorySeparator + oaCfgFileName;
   {$ENDIF}
   {$IFDEF UNIX} {LINUX and UNICES}
   path := IncludeTrailingPathDelimiter(GetEnvironmentVariable('HOME')) +
      '.openarena' + DirectorySeparator +
      'baseoa' + DirectorySeparator + oaCfgFileName;
   {$ENDIF}
   {$IFDEF DARWIN} {MACOSX}
   {$ENDIF}

   result := path;
end;

function oaCfgFileExists(const fn: string): boolean;
begin
   result := FileExists(fn);
end;

procedure CheckOAConfig();
var
   exists, defaultexists: boolean;

begin
   exists := oaCfgFileExists(oacfgfile);
   if(not exists) then begin
      oacfgfile := oaGetCfgFile();

      defaultexists := oaCfgFileExists(oacfgfile);

      if(defaultexists) then begin
         MessageDlg('Warning', 'The previously selected OpenArena config file was not found, '+
         'so default one was selected.',
            mtWarning, [mbOK], 0);
      end else
         MessageDlg('Warning', 'No OA config file found.',
            mtWarning, [mbOK], 0);
   end;

   oaScanMods();
end;

{save playername to OA config}
procedure OAConfigLoad();
var
   f: text;
   l: string;
   buf1: array[0..1023] of byte;
   err: longint;

begin
   {open the oa config file}
   Assign(f, oacfgfile);
   SetTextBuf(f, buf1, 1024);

   Reset(f);
   err := IOResult();
   if(err = 0) then begin
      {go through the oa cfg file}
      repeat
         readln(f, l);
         err := IOResult();
         if(err = 0) then begin
            if(pos('seta name', l) <> 0) then begin
               {get the player name}
               delete(l, 1, Length('seta name "'));
               playername := copy(l, 1, Length(l)-1);
               break;
            end;
            err := IOResult();
            if(err <> 0) then break;
         end else break;
      until eof(f);

      {close both files}
      Close(f);
      IOResult();
   end;
end;

function OAConfigSave(const fn: string): boolean;
const
   errtitle = 'OA Config File Error';
   nochgstr = 'No changes were made.';

var
   f, nf: text;
   l, nfn, bakfn: string;
   buf1, buf2: array[0..1023] of byte;
   err: longint;

begin
   result := false;

   {create a new file}
   nfn := IncludeTrailingPathDelimiter(ExtractFilePath(fn))+'temp.new';
   Assign(nf, nfn);
   Rewrite(nf);
   SetTextBuf(nf, buf1, 1024);

   {open the oa config file}
   Assign(f, fn);
   SetTextBuf(f, buf2, 1024);

   Reset(f);
   err := IOResult();
   if(err = 0) then begin

      {go through the oa cfg file}
      repeat
         readln(f, l);
         err := IOResult();
         if(err = 0) then begin
            if(pos('seta name', l) <> 0) then begin
               writeln(nf, 'seta name "'+playername+'"');
            end else begin
               writeln(nf, l);
            end;
            err := IOResult();
            if(err <> 0) then begin
               MessageDlg(errtitle, 'A new file could not be written for: '+fn+'. '+nochgstr,
                  mtError, [mbOK], 0);
               break;
            end;
         end else begin
            MessageDlg(errtitle, ' The config file could not be read.'+nochgstr,
               mtError, [mbOK], 0);
            break;
         end;
      until eof(f);

      {close both files}
      Close(f);
      err := IOResult();
      if(err <> 0) then begin
         MessageDlg(errtitle, 'The config file: '+fn+' could not be closed.'+nochgstr,
            mtError, [mbOK], 0);
      end;

      Close(nf);
      err := IOResult();
      if(err <> 0) then begin
         MessageDlg(errtitle, 'The new OA config file could not be closed for: '+fn+'. '+nochgstr,
            mtError, [mbOK], 0);
         exit;
      end;

      {get the name of the backup file}
      bakfn := IncludeTrailingPathDelimiter(ExtractFilePath(fn))+DirectorySeparator+ExtractFilename(fn)+'.bak';

      {erase the old backup file}
      assign(f, bakfn);
      erase(f);
      IOResult();

      {rename the oa config file to backup file}
      assign(f, fn);
      rename(f, bakfn);
      IOResult();

      {rename the new file to oa config file}
      rename(nf, fn);
      err := IOResult();
      if(IOResult() <> 0) then begin
         MessageDlg(errtitle, 'Could not set the new file for: '+fn+' as OA config file. The old file was preserved as: '+bakfn,
            mtError, [mbOK], 0);
         exit;
      end;
   end else
      MessageDlg(errtitle, ' The file: '+fn+' cannot be opened.'+
      'Please set a valid config file in OANC configuration options.', mtError, [mbOK], 0);

   if(err = 0) then result := true;
end;

{MODS}
procedure oaSaveToMods();
var
   path, cf: string; {base path, config file}
   i: longint;
   err: boolean = false; {indicates if an error condition was encountered}
   errs: string; {contains a list of files for which there were errors}

begin
   path := ExcludeTrailingPathDelimiter(oaBasePath());
   {if mods were selected}
   if(modlistsel.Count > 0) then begin
      for i := 0 to (modlistsel.Count-1) do begin
         cf := path+DirectorySeparator+modlistsel[i]+DirectorySeparator+oaCfgFileName;

         {save the config file}
         if(OAConfigSave(cf) = false) then begin
            err := true;
            errs := errs + cf + #10;
         end;
      end;
      

      {message}
      if(not err) then
         MessageDlg('Success', 'All config files saved successfully.', mtInformation, [mbOK], 0)
      else
         MessageDlg('Error', 'Error saving config files: '+#10+errs, mtError, [mbOK], 0);
   {if no mods then try to write directly to config file}
   end else begin
      if(OAConfigSave(oacfgfile)) then
         MessageDlg('Success', 'OpenArena configuration file saved successfully.', mtInformation, [mbOK], 0);
   end;
end;

procedure oaScanMods();
var
   src: TSearchRec;
   result: longint;
   path, cf: string;

begin
   {we need to find every directory in a directory below baseoa, where oa config is}
   path := oaBasePath();

   {create a new mod list or clear existing one}
   modlist.Clear();

   {find first}
   result := FindFirst(path+'*', faReadOnly or faDirectory, src);
   if(result = 0) then begin
      repeat
         if(src.Attr and faDirectory > 0) then begin
            if(src.Name <> '.') and (src.Name <> '..') then begin
               cf := path+DirectorySeparator+src.Name+DirectorySeparator+oaCfgFileName;
               {only add potential mods if there seems to be a config file}
               if(FileExists(cf)) then begin
                  modlist.Add(src.Name);
               end;
            end;
         end;
         result := FindNext(src);
      until (result <> 0);
   end;

   {we're done}
   FindClose(src);
end;

function oaHasMods(): boolean;
begin
   result := false;
   if(modlist.Count > 0) then result := true;
end;

function oaBasePath(): string;
begin
   result := ExtractFilePath(ExcludeTrailingPathDelimiter(ExtractFilePath(oacfgfile)));
end;

{ RENDERING }

procedure PreviewName(const thename: string; const shp: TShape);
var
   n, i, px, py: longint;
   c, nc: byte;
   tmpimg: TBGRABitmap;
   p: TBGRAPixel;
   pclr: longint = CLR_WHITE;
   name, normalname: string;

procedure setcolor();
begin
   p.red := colors[pclr][0];
   p.green := colors[pclr][1];
   p.blue := colors[pclr][2];
   p.alpha := 255;
end;

function normalchar(): boolean;
begin
   result := true;
   c := byte(name[i]);
   if(i < n) then
      nc := byte(name[i+1])
   else
      nc := 0;

   {check for a color character}
   if(c = byte('^')) then begin
      {we can only use colors from 48(0) to 56(8)}
      if(nc >= 48) and (nc <= 56) then begin
         pclr := nc - 48;

         setcolor();
         inc(i, 2); result := false;
      end;
   end;
end;

procedure getnormalname();
begin
   n := length(name);
   i := 1;
   if(n > 0) then repeat
      if(normalchar()) then begin
         normalname := normalname + name[i];
         inc(i);
      end;
   until (i > n);
end;

begin
   if(thename <> '') then begin
      name := thename;
      normalname := '';

      {get name}
      getnormalname();

      {calculate central positions}
      px := (shp.Width div 2) - (length(normalname)*CHAR_WIDTH div 2);
      py := shp.Height div 2 - (CHAR_HEIGHT div 2);

      {set initial color}
      pclr := CLR_WHITE; setcolor();

      if(n > 0) then begin
         {need a new temporary image}
         tmpimg := TBGRABitmap.Create(CHAR_WIDTH, CHAR_HEIGHT);

         {start going through the text}
         i := 1;
         repeat
            if(i > n) then break;

            {get the current and next characters}
            if(not normalchar()) then continue;

            {we can only draw a character if it exists in the image table}
            if(c <= MAX_CHAR) then begin
               {fill the temporary image with desired color and multiply}
               tmpimg.Fill(p);
               tmpimg.BlendImage(0, 0, imgs[c], boMultiply);

               {draw image}
               tmpimg.Draw(shp.Canvas, px, py, false);
            end;

            {move on to the next character}
            px := px + CHAR_WIDTH;
            inc(i);
         until (i > n);
      end;
   end;
end;

INITIALIZATION
   modlist := TStringList.Create();
   modlistsel := TStringList.Create();

   oacfgfile := oaGetCfgFile();
   {$IFDEF WINDOWS}
   localcfgpath := ExcludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)))+DirectorySeparator;
   {$ENDIF}

END.
